@@ -42,13 +42,13 @@
 #include <QtQuick/qquickview.h>
 #include <QtQuick/private/qquickitem_p.h>
 #include <QtGui/private/qguiapplication_p.h>
-#include <QtLabsTemplates/private/qquickapplicationwindow_p.h>
-#include <QtLabsTemplates/private/qquickoverlay_p.h>
-#include <QtLabsTemplates/private/qquickcontrol_p.h>
-#include <QtLabsTemplates/private/qquicklabel_p.h>
-#include <QtLabsTemplates/private/qquicktextarea_p.h>
-#include <QtLabsTemplates/private/qquicktextfield_p.h>
-#include <QtLabsControls/private/qquickproxytheme_p.h>
+#include <QtQuickTemplates2/private/qquickapplicationwindow_p.h>
+#include <QtQuickTemplates2/private/qquickoverlay_p.h>
+#include <QtQuickTemplates2/private/qquickcontrol_p.h>
+#include <QtQuickTemplates2/private/qquicklabel_p.h>
+#include <QtQuickTemplates2/private/qquicktextarea_p.h>
+#include <QtQuickTemplates2/private/qquicktextfield_p.h>
+#include <QtQuickControls2/private/qquickproxytheme_p.h>
 #include "../shared/util.h"
 #include "../shared/visualtestutil.h"
 
@@ -71,6 +71,7 @@ private slots:
     void locale();
     void activeFocusControl_data();
     void activeFocusControl();
+    void focusAfterPopupClosed();
 };
 
 void tst_applicationwindow::qmlCreation()
@@ -420,7 +421,7 @@ void tst_applicationwindow::attachedProperties()
     QCOMPARE(childItem->property("attached_footer").value<QQuickItem *>(), childAppWindow->footer());
     QCOMPARE(childItem->property("attached_overlay").value<QQuickItem *>(), childAppWindow->overlay());
 
-    childControl->setParentItem(Q_NULLPTR);
+    childControl->setParentItem(nullptr);
     QVERIFY(!childControl->window());
     QVERIFY(!childControl->property("attached_window").value<QQuickApplicationWindow *>());
     QVERIFY(!childControl->property("attached_contentItem").value<QQuickItem *>());
@@ -429,7 +430,7 @@ void tst_applicationwindow::attachedProperties()
     QVERIFY(!childControl->property("attached_footer").value<QQuickItem *>());
     QVERIFY(!childControl->property("attached_overlay").value<QQuickItem *>());
 
-    childItem->setParentItem(Q_NULLPTR);
+    childItem->setParentItem(nullptr);
     QVERIFY(!childItem->window());
     QVERIFY(!childItem->property("attached_window").value<QQuickApplicationWindow *>());
     QVERIFY(!childItem->property("attached_contentItem").value<QQuickItem *>());
@@ -505,9 +506,8 @@ class TestTheme :  public QQuickProxyTheme
 public:
     TestTheme(QPlatformTheme *theme) : QQuickProxyTheme(theme), m_font("Courier")
     { QGuiApplicationPrivate::platform_theme = this; }
-    ~TestTheme() { QGuiApplicationPrivate::platform_theme = theme(); }
 
-    const QFont *font(Font type = SystemFont) const Q_DECL_OVERRIDE
+    const QFont *font(Font type = SystemFont) const override
     {
         Q_UNUSED(type);
         return &m_font;
@@ -522,7 +522,7 @@ void tst_applicationwindow::defaultFont()
 
     QQmlEngine engine;
     QQmlComponent component(&engine);
-    component.setData("import Qt.labs.controls 1.0; ApplicationWindow { }", QUrl());
+    component.setData("import QtQuick.Controls 2.0; ApplicationWindow { }", QUrl());
 
     QScopedPointer<QQuickApplicationWindow> window;
     window.reset(static_cast<QQuickApplicationWindow *>(component.create()));
@@ -640,6 +640,50 @@ void tst_applicationwindow::activeFocusControl()
     QCOMPARE(window->activeFocusControl(), activeFocusControl);
 }
 
+void tst_applicationwindow::focusAfterPopupClosed()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine);
+    component.loadUrl(testFileUrl("focusAfterPopupClosed.qml"));
+    QScopedPointer<QQuickWindow> window(qobject_cast<QQuickWindow*>(component.create()));
+    QVERIFY(window);
+
+    window->show();
+    window->requestActivate();
+    QVERIFY(QTest::qWaitForWindowActive(window.data()));
+    QVERIFY(QGuiApplication::focusWindow() == window.data());
+
+    QQuickItem* contentItem = window->contentItem();
+    QVERIFY(contentItem);
+    QVERIFY(contentItem->hasActiveFocus());
+
+    QQuickItem* focusScope = window->property("focusScope").value<QQuickItem*>();
+    QVERIFY(focusScope);
+    QVERIFY(focusScope->hasActiveFocus());
+
+    QSignalSpy spy(window.data(), SIGNAL(focusScopeKeyPressed()));
+    QTest::keyClick(window.data(), Qt::Key_Space);
+    QCOMPARE(spy.count(), 1);
+
+    // Open the menu.
+    QQuickItem* toolButton = window->property("toolButton").value<QQuickItem*>();
+    QVERIFY(toolButton);
+    QTest::mouseClick(window.data(), Qt::LeftButton, Qt::NoModifier,
+        toolButton->mapFromScene(QPointF(toolButton->width() / 2, toolButton->height() / 2)).toPoint());
+    QVERIFY(!focusScope->hasActiveFocus());
+
+    // The FocusScope shouldn't receive any key events while the menu is open.
+    QTest::keyClick(window.data(), Qt::Key_Space);
+    QCOMPARE(spy.count(), 1);
+
+    // Close the menu. The FocusScope should regain focus.
+    QTest::keyClick(window.data(), Qt::Key_Escape);
+    QVERIFY(focusScope->hasActiveFocus());
+
+    QTest::keyClick(window.data(), Qt::Key_Space);
+    QCOMPARE(spy.count(), 2);
+}
+
 QTEST_MAIN(tst_applicationwindow)
 
 #include "tst_applicationwindow.moc"
